// SQL10.cpp : main project file.

using namespace System;

/* Raima::Rdm is the system module that contains the ADO.NET
 * data provider for RDM */
using namespace Raima::Rdm;

/* EXAMPLE - SQL10 for C++ using ADO.NET */
/* This example shows multi-user access to a database.  Two connections are
 * established using shared mode.  In one data is inserted into the database.
 * The other one also opens the database.  The second connection is used to
 * read the data while the first connection still has a transaction open,
 * which fails with a timeout.  Then the transaction is committed (freeing
 * the locks) and the read using the second connection then succeeds.
 * 
 * The following methods are illustrated:
 *     Raima::Rdm::RdmConnection:
 *         BeginTransaction() Method
 *         ChangeDatabase(string) Method
 *         Close() Method
 *         CreateCommand() Method
 *         Open() Method
 *         RdmConnection() Constructor
 *     Raima::Rdm::RdmCommand:
 *         CommandText Property
 *         ExecuteNonQuery() Method
 *         ExecuteReader() Method
 *         Parameters Property
 *     Raima::Rdm::RdmDataReader:
 *         Item[int] Property
 *         Read() Method
 *     Raima::Rdm::RdmTransaction:
 *         Commit() Method
 * 
 * They are fully document in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown execptions
 * but good programming practices would dictate that this occur in
 * a full application.
 */

static void DropDatabase(
    RdmConnection ^conn,
    String        ^name)
{
    RdmCommand ^cmd = conn->CreateCommand();

    /* Attempt to drop this database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    try
    {
        cmd->CommandText = "DROP DATABASE " + name;
        cmd->ExecuteNonQuery();

        cmd->CommandText = "COMMIT";
        cmd->ExecuteNonQuery();
    }
    catch (Exception ^)
    {
    }
}

static void CreateThisDatabase(
    RdmConnection   ^conn,
    String          ^name,
    array<String ^> ^cmds)
{
    RdmCommand ^cmd = conn->CreateCommand();

    cmd->CommandText = "CREATE DATABASE " + name;
    cmd->ExecuteNonQuery();

    for (int ii = 0; ii < cmds->Length; ii++)
    {
        cmd->CommandText = cmds[ii];
        cmd->ExecuteNonQuery();
    }

    cmd->CommandText = "COMMIT";
    cmd->ExecuteNonQuery();

    cmd->CommandText = "CLOSE " + name;
    cmd->ExecuteNonQuery();
}

static void CreateDatabases(
    RdmConnection ^conn)
{
    array<String ^> ^cmds = {
        "CREATE TABLE Table1 (id INTEGER PRIMARY KEY)"
    };

    /* Drop any existing copy of the databases. */
    DropDatabase(conn, "SQL10");

    /* Attempt to create the databases */
    CreateThisDatabase(conn, "SQL10", cmds);
}

int main(array<System::String ^> ^args)
{
    /* Create the connection with a simple connection string */
    RdmConnection ^conn1 = gcnew RdmConnection("host=local;openmode=s");
    conn1->Open();

    CreateDatabases(conn1);

    /* Open the database */
    conn1->ChangeDatabase("SQL10");

    /* Create a second connection */
    RdmConnection ^conn2 = gcnew RdmConnection("host=local;database=SQL10;openmode=s;timeout=2");
    conn2->Open();

    /* Create a command in connection 1 and one in connection 2 */
    RdmCommand ^cmd1 = conn1->CreateCommand();
    RdmCommand ^cmd2 = conn2->CreateCommand();

    /* Start a new transaction in connection 1 */
    RdmTransaction ^trans = conn1->BeginTransaction();

    /* Insert three rows into the table using connection 1 */
    for (int ii = 1; ii < 4; ii++)
    {
        cmd1->CommandText = "INSERT INTO Table1 VALUES " + ii;
        cmd1->ExecuteNonQuery();
    }

    /* Attempt to read the data from connection 2, expect a timeout
     * because connection 1 is holding locks as the transaction is 
     * still active */
    try
    {
        cmd2->CommandText = "SELECT id FROM Table1";
        RdmDataReader ^rdr = cmd2->ExecuteReader();

        while (rdr->Read())
            Console::WriteLine("Should not get this data - " + rdr[0]);
    }
    catch (RdmException ^e)
    {
        /* Double-check to make sure the error is a timeout */
        if (e->Message == "timeout expired")
            Console::WriteLine("Received expected timeout");
        else
            Console::WriteLine("Unexpected exception: " + e);
    }

    /* Commit the transaction */
    trans->Commit();

    /* Attempt to read the data again, which should now succeed */
    cmd2->CommandText = "SELECT id FROM Table1";
    RdmDataReader ^reader = cmd2->ExecuteReader();

    while (reader->Read())
        Console::WriteLine(reader[0]);

    conn1->Close();
    conn2->Close();

    Console::WriteLine("Hit <Enter> to continue");
    Console::ReadLine();
    return 0;
}
